package com.diagbot.repository;

import com.diagbot.dto.DrugBillNeoDTO;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.entity.node.Gender;
import com.diagbot.entity.node.Group;
import com.diagbot.entity.node.MedAllergen;
import com.diagbot.entity.node.MedCodeName;
import com.diagbot.entity.node.MedRegName;
import com.diagbot.entity.node.Medicine;
import com.diagbot.entity.node.Vital;
import com.diagbot.entity.relationship.MedRegNameGender;
import com.diagbot.entity.relationship.MedRegNameGroup;
import com.diagbot.entity.relationship.MedRegNameMedAllergen;
import com.diagbot.entity.relationship.MedRegNameMedRegName;
import com.diagbot.entity.relationship.MedRegNameVital;
import com.diagbot.util.NeoUtil;
import com.diagbot.vo.Drug;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

@Component
public class MedRegNameNode {

    @Autowired
    GroupRepository groupRepository;
    @Autowired
    MedAllergenRepository medAllergenRepository;
    @Autowired
    VitalRepository vitalRepository;
    @Autowired
    MedRegNameRepository medRegNameRepository;
    @Autowired
    GenderRepository genderRepository;

    /**
     * 获取高危药品数据
     */
    public List<HighRiskNeoDTO> getHighRiskDrug(List<Drug> druglist, MedicineRepository medRepository,
                                                MedicineCodeRepository medCodeRepository) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();
        HighRiskNeoDTO highRiskNeoDTO;
        String drugname, drugform;
        List<Medicine> medicine;
        List<MedCodeName> medcdnames;
        //        MedRegName medreg;

        // Long t1 = System.currentTimeMillis();

        for (Drug drug : druglist) {
            highRiskNeoDTO = new HighRiskNeoDTO();

            drugname = drug.getUniqueName();
            highRiskNeoDTO.setName(drug.getName());
            highRiskNeoDTO.setStandname(drugname);
            drugform = drug.getForm();

            medicine = medRepository.findByNameIs(drugname);

            if (null != medicine) {
                for (Medicine med : medicine) {
                    List<MedRegName> MedRegNames = medRepository.findMedCodeNamebyDosage(med.getName(), drugform);
                    for (MedRegName medreg : MedRegNames) {
                        if (NeoUtil.isExist(medreg) && null != medreg.getRisklevel() && medreg.getRisklevel().trim().length() > 0) {
                            highRiskNeoDTO.setIsHighrisk(true);
                            highRiskNeoDTO.setHighLevel(medreg.getRisklevel());
                            highRiskNeoDTO.setType(Constants.yaoping);
                            highRiskNeoDTO.setForm(drugform);
                            highRiskNeoDTOS.add(highRiskNeoDTO);
                            break;
                        }
                    }
                }
            }
        }
        // Long t2 = System.currentTimeMillis();
        //
        // System.out.println((t2-t1)/1000.0);

        return highRiskNeoDTOS;
    }


    public DrugBillNeoDTO MedRegNametoDrugDTO(MedRegName medRegName, DrugBillNeoDTO drugBillNeoDTO) {

        NodeInfo nodeInfo;
        Set<MedRegNameVital> medVitals = medRegName.getMedRegNameVital();
        for (MedRegNameVital medvital : medVitals) {
            if (NeoUtil.isExist(medvital)) {
                if (NeoUtil.isExist(medvital.getVital())) {
                    nodeInfo = new NodeInfo();
                    Vital vital = medvital.getVital();
                    String name = (vital.getVitalidx() == null) ? vital.getName() : vital.getVitalidx();
                    nodeInfo.setName(name);
                    nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(vital.getMinval()));
                    nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(vital.getMaxval()));
                    nodeInfo.setUnit(vital.getUnit());
                    nodeInfo.setRange(vital.getRange());
                    nodeInfo.setTypeval(name);
                    //                    NodeNeoDTO nodeNeoDTO = NeoUtil.fillNodeNeoDTO(medvital.getVital());
                    if (name.equals(Constants.age)) {
                        drugBillNeoDTO.setAgeNeoDTO(NeoUtil.updateNodeInfo(nodeInfo));
                    } else {
                        drugBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(nodeInfo));
                    }
                }
            }
        }


        Set<MedRegNameGroup> medGroups = medRegName.getMedRegNameGroup();
        for (MedRegNameGroup medgroup : medGroups) {
            if (NeoUtil.isExist(medgroup) && NeoUtil.isExist(medgroup.getGroup())
                    && !NeoUtil.inlist(medgroup.getGroup().getName(), drugBillNeoDTO.getGroup())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medgroup.getGroup().getName());
                nodeNeoDTO.setTermtype(Constants.group);
                drugBillNeoDTO.getGroup().add(nodeNeoDTO);
            }
        }


        Set<MedRegNameMedAllergen> medAllergens = medRegName.getMedRegNameMedAllergens();
        for (MedRegNameMedAllergen medAllergen : medAllergens) {
            if (NeoUtil.isExist(medAllergen) && NeoUtil.isExist(medAllergen.getMedAllergen())
                    && !NeoUtil.inlist(medAllergen.getMedAllergen().getName(), drugBillNeoDTO.getAllergicmeds())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medAllergen.getMedAllergen().getName());
                nodeNeoDTO.setTermtype(medAllergen.getMedAllergen().getMedtype());
                drugBillNeoDTO.getAllergicmeds().add(nodeNeoDTO);
            }
        }


        Set<MedRegNameMedRegName> medRegmedRegNames = medRegName.getMedRegNameMedregnames_out();
        for (MedRegNameMedRegName medRegnm : medRegmedRegNames) {
            if (NeoUtil.isExist(medRegnm) && NeoUtil.isExist(medRegnm.getMedRegName())
                    && !NeoUtil.inlist(medRegnm.getMedRegName().getName(), drugBillNeoDTO.getAllergicmeds())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medRegnm.getMedRegName().getName());
                nodeNeoDTO.setTermtype(Constants.yaoping);
                drugBillNeoDTO.getAllergicmeds().add(nodeNeoDTO);
            }
        }

        /*String conflict_gender = medRegName.getConflict_gender();
        if (null != conflict_gender) {
            NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            nodeNeoDTO.setName(conflict_gender);
            nodeNeoDTO.setTermtype(Constants.gender);
            drugBillNeoDTO.setGender(nodeNeoDTO);
        }*/

        MedRegNameGender medRegNameGender = medRegName.getMedRegNameGender();
        if (NeoUtil.isExist(medRegNameGender)) {
            if (NeoUtil.isExist(medRegNameGender.getGender())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medRegNameGender.getGender().getName());
                nodeNeoDTO.setTermtype(Constants.gender);
                drugBillNeoDTO.setGender(nodeNeoDTO);
            }
        }

        String conflict_geiyao = medRegName.getConflict_geiyao();
        if (null != conflict_geiyao) {
            NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            nodeNeoDTO.setName(conflict_geiyao);
            nodeNeoDTO.setTermtype(Constants.geiyao);
            drugBillNeoDTO.setGeiyaoNeoDTO(nodeNeoDTO);
        }


        return drugBillNeoDTO;
    }

    /**
     * 更新关系状态属性
     */
    public void updateRelationExt(MedRegName medRegName, String relname, Integer status, Integer ruleType, String e_name, MedRegNameRepository medRegNameRepository) {
        Boolean isExist = false;
        try {
            switch (relname) {
                case "药品注册名称禁忌性别":
                    Gender gender = genderRepository.findByNameIs(e_name).get(0);
                    MedRegNameGender medRegNameGender = new MedRegNameGender();
                    medRegNameGender.setGender(gender);
                    medRegNameGender.setMedRegName(medRegName);
                    medRegNameGender.setStatus(status);
                    medRegNameGender.setRuleType(ruleType);
                    medRegName.setMedRegNameGender(medRegNameGender);
                    break;
                case "药品注册名称禁忌禁忌人群":
                    Group group = groupRepository.findByNameIs(e_name).get(0);
                    Set<MedRegNameGroup> medRegNameGroupSet = medRegName.getMedRegNameGroup();
                    isExist = false;
                    for (MedRegNameGroup item : medRegNameGroupSet) {
                        if (item.getGroup() != null
                                && item.getGroup().getName().equals(e_name)) {
                            isExist = true;
                            item.setMedRegName(medRegName);
                            item.setGroup(group);
                            item.setStatus(status);
                            item.setRuleType(ruleType);
                        }
                    }
                    if (!isExist) {
                        MedRegNameGroup medRegNameGroup = new MedRegNameGroup();
                        medRegNameGroup.setGroup(group);
                        medRegNameGroup.setMedRegName(medRegName);
                        medRegNameGroup.setStatus(status);
                        medRegNameGroup.setRuleType(ruleType);
                        medRegNameGroupSet.add(medRegNameGroup);
                    }
                    medRegName.setMedRegNameGroup(medRegNameGroupSet);
                    break;
                case "药品注册名称禁忌药物过敏原":
                    MedAllergen medAllergen = medAllergenRepository.findByNameIs(e_name).get(0);
                    Set<MedRegNameMedAllergen> medRegNameMedAllergenSet = medRegName.getMedRegNameMedAllergens();
                    isExist = false;
                    for (MedRegNameMedAllergen item : medRegNameMedAllergenSet) {
                        if (item.getMedAllergen() != null
                                && item.getMedAllergen().getName().equals(e_name)) {
                            isExist = true;
                            item.setMedRegName(medRegName);
                            item.setMedAllergen(medAllergen);
                            item.setStatus(status);
                            item.setRuleType(ruleType);
                        }
                    }
                    if (!isExist) {
                        MedRegNameMedAllergen medRegNameMedAllergen = new MedRegNameMedAllergen();
                        medRegNameMedAllergen.setMedAllergen(medAllergen);
                        medRegNameMedAllergen.setMedRegName(medRegName);
                        medRegNameMedAllergen.setStatus(status);
                        medRegNameMedAllergen.setRuleType(ruleType);
                        medRegNameMedAllergenSet.add(medRegNameMedAllergen);
                    }
                    medRegName.setMedRegNameMedAllergens(medRegNameMedAllergenSet);
                    break;
                case "药品注册名称禁忌药品注册名称":
                    break;
                case "药品注册名称禁忌体征":
                    Vital vital = vitalRepository.findByNameIs(e_name).get(0);
                    Set<MedRegNameVital> medRegNameVitalSet = medRegName.getMedRegNameVital();
                    isExist = false;
                    for (MedRegNameVital item : medRegNameVitalSet) {
                        if (item.getVital() != null
                                && item.getVital().getName().equals(e_name)) {
                            isExist = true;
                            item.setMedRegName(medRegName);
                            item.setVital(vital);
                            item.setStatus(status);
                            item.setRuleType(ruleType);
                        }
                    }
                    if (!isExist) {
                        MedRegNameVital medRegNameVital = new MedRegNameVital();
                        medRegNameVital.setVital(vital);
                        medRegNameVital.setMedRegName(medRegName);
                        medRegNameVital.setStatus(status);
                        medRegNameVital.setRuleType(ruleType);
                        medRegNameVitalSet.add(medRegNameVital);
                    }
                    medRegName.setMedRegNameVital(medRegNameVitalSet);
                    break;
            }
            medRegNameRepository.save(medRegName, 1);
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {

        }
    }
}